var sanitize = require('validator').sanitize;
var rooms = require('../rooms.js').rooms;


var fs = require('fs');
var mongoose = require('mongoose');
var db = mongoose.connection;
var Room;

mongoose.connect('mongodb://localhost/2406rooms');
db.on('error', console.error.bind(console, 'MongoDB connection error:'));

var setupRoomDB = function () {
    var roomSchema = mongoose.Schema({
	name: {type: String, index: true, unique: true},
	title: String,
	description: String,
	roomExits: Array
    });
    Room = mongoose.model("2406rooms", roomSchema);
    //empty the database
    emptyDB(function(){		//give it the following callback which fills in each room
		//make the start room first...
		new Room( { 	name: "start",
						title: "Starting Room",
						description: "This is the starting room of your adventure game! What mysteries could possibly lay ahead?",
						roomExits: ['bridge']} ).save(console.log("start room created.."));
		//load the old rooms.js file
		for (var r in rooms){
			var newRoom = new Room({ 	name: r,							//name of the room
										title: rooms[r].title,				//must access properties!
										description: rooms[r].description,
										roomExits: rooms[r].roomExits	});
		    newRoom.save( function(err, room) { 
				if (err) {
					console.log("Error adding room: " + err);
				}else{
					console.log("room created.." + room.name + ", title: " + room.title);
				}
			});
		}}); 
}
db.once('open', setupRoomDB);


var emptyDB = function(callback){
	Room.remove({}, function(err) { 
	if (err) {
	    res.send("Error: Couldn't delete all existing documents");
	    return;
	} else {
	    console.log("All documents removed.");
	    callback();	//call the call back!
	}
})};


function index(req, res) {
    if (req.session.player) {
        res.redirect("/game");
    } else {
	res.render('index', { title: 'COMP 2406 Adventure Demo', 
			      error: req.query.error });
    }
}

function start(req, res) {
    var player = req.body.player;
    // Sanitize the player name as it may be displayed
    req.session.player = sanitize(player).escape();
    req.session.currentRoom = "start"; // We start at the start now.......
    res.redirect("/game")
}

function quit(req, res) {
    req.session.destroy(function(err){
        if(err){
            console.log("Error: %s", err);
        }
    });
    res.redirect("/");
}

function game(req, res) {
    if (req.session.player) {
		res.render("room.jade", {title: "TITLE SPACE"});
    } else {
        res.redirect("/");
    }
}

function doAction(req, res) {
    var action = req.body.action;
    var room = req.body.room;    

	//console.log("yolo log");
    if (action === "move") {
		req.session.currentRoom = room;
		res.send("Success");
    }
    else {
	res.send("Error: InvalidAction");
    }
}

function getContents(req, res) {    
	var currentRoom = req.session.currentRoom;
		Room.findOne({name: currentRoom},function(err, room){
			if (err || room === []) {
				res.send("Error fetching current room.");
			}
			else {
				room.description = room.description.replace("<?player>",req.session.player);
				res.send(room);
			}});
}


function myEditor(req,res){
	Room.find({}, 'name title description roomExits', function(err, rooms){
		if(err){
			res.redirect('/');	//redirect to home page if err
		}
		else{
			res.render('editor.jade', {msg: "", rooms: rooms});
		}
	});

}

function edit(req,res){
	
	//find all rooms
	Room.find(function(err, rooms){
			if(err){
				res.redirect('/');				//redirect to home page if err
			}
			else{
				var oldRooms = rooms;			//store rooms as old rooms variables					
				var type = req.body.button;
				switch(type)					//switch statement checks which type of edit being made.
				{
					case "edit":
						console.log("Taking action: " + type);
						//scoop form elements
						var temp_id = req.body.id;				//the id of the element. Note: this is hidden on client side html.
						var temp_name = req.body.roomName;
						var temp_titl = req.body.roomTitle;
						var temp_desc = req.body.roomDescription;
						
						//if invalid fields..
						if (temp_name === "" || temp_titl === "" || temp_desc === ""){
							res.render('editor.jade', {msg: "Error: Edit contained invalid fields", rooms: oldRooms});
						}
						//otherwise, update the field.
						else{
							Room.findById(temp_id, function(err, data){
								if (err){
									res.render('editor.jade', {msg: "Error: Unable to find room being edited. This indicates a glitch in the server matrix...", rooms: oldRooms});
								}
								else
								{
									//make changes to the room data
									data.name = temp_name;
									data.title = temp_titl;
									data.description = temp_desc;
									//save this data.
									data.save(function(err){
										if(err){
											res.render('editor.jade', {msg: "Error: Unable to update room edit..", rooms: oldRooms});
										}
										else{
											//return user to the editor with new rooms
											Room.find(function(err,newRooms){
												if(err){
													res.redirect('/');
												}
												else{
													res.render('editor.jade', {msg: "Success! Room edited", rooms: newRooms});
												}											
											});									
										}
									});
								}
							});
							
						}

						break;
					case "remove":
						console.log("Taking action: " + type);
						var temp_id = req.body.id;				//the id of the room to be removed..
						Room.findById(temp_id, function(err, oldRoom){
							if(err){
								res.redirect('/editor');
							}
							else{
								//now we have the old room that we are about to remove..							
								var oldRoomName = oldRoom.name;
								var listOfRooms = "";
								//for each room in the array..
								oldRooms.forEach(function(value, index){
									if(value.name !== oldRoomName){
										value.roomExits.forEach(function(value2,index2){
											if(value2 === oldRoomName){
												listOfRooms = listOfRooms + " > " + value.name;
											}
												
										})
									}
								});
								//Not sure if the callbacks using for each will cause a race condition with
								//the message being rendered in this...
								Room.findByIdAndRemove(temp_id, function(err){
									if(err){
										console.log("Error adding removing room: " + err);
										res.redirect('/');
									}
									else{
										Room.find(function(err,newRooms){
													if(err){
														res.redirect('/');
													}
													else{
														res.render('editor.jade', {msg: "Success! Room: " + oldRoomName + " removed. Please revise the exits in the following room: " + listOfRooms + ".", rooms: newRooms});
													}											
												});	
									}
									
								});
								
							}
							
						});
						
						break;
						
					case "add":
						var temp_name = req.body.roomName;
						var temp_titl = req.body.roomTitle;
						var temp_desc = req.body.roomDescription;
						if (temp_name === "" || temp_titl === "" || temp_desc === ""){
							res.render('editor.jade', {msg: "Error: Added room must not have empty fields", rooms: oldRooms});
						}
						else{
							//try finding a room with this name..
							Room.findOne({name: temp_name}, function(err, data){
								if(err){
									res.redirect('/editor');
								}
								else
								{
									if(!data){ //if we don't find any existing rooms with name temp_name, add it to the database
										var newRoom = new Room({ 	name: temp_name,							//name of the room
																	title: temp_titl,				//must access properties!
																	description: temp_desc,
																	roomExits: []	});
										newRoom.save( function(err, room) { 
											if (err) {
												console.log("Error adding room: " + err);
												res.redirect('/');
											}else{
												console.log("room created.." + room.name + ", title: " + room.title);
												//return user to the editor with new rooms
												Room.find(function(err,newRooms){
													if(err){
														res.redirect('/');
													}
													else{
														res.render('editor.jade', {msg: "Success! Room: " + temp_name + " added!", rooms: newRooms});
													}											
												});		
											}
										});
										
										
									}
									else //otherwise, 
									{
										res.render('editor.jade', {msg: "Error: Cannot add the same room twice!", rooms: oldRooms});							
									}
									
								}
							});	
						}
						break;
					
					case "removeExit":
						var temp_id = req.body.id;				//id of the room we are removing exit from
						var temp_exit = req.body.roomExit;		//exit that we are removing..
						Room.findById(temp_id, function(err, data){	//find the room
								if (err){
									res.render('editor.jade', {msg: "Error: Unable to find room. This indicates a glitch in the server matrix...", rooms: oldRooms});
								}
								else
								{
									//remove the tempExit from the roomExits array
									var idx = data.roomExits.indexOf(temp_exit);
									if (idx != -1){
										data.roomExits.splice(idx,1);	//splice out the exit
									}
									else{ //if its not an element in the array, this is bad. means others are editing rooms at same time..
										res.render('editor.jade', {msg: "Error: Cannot delete exit which is no longer a room exit..", rooms: oldRooms});
									}
									//save this data.
									data.save(function(err){
										if(err){
											res.render('editor.jade', {msg: "Error: Unable to remove room exit..", rooms: oldRooms});
										}
										else{
											//return user to the editor with new rooms
											Room.find(function(err,newRooms){
												if(err){
													res.redirect('/');
												}
												else{
													res.render('editor.jade', {msg: "Success! room exit removed", rooms: newRooms});
												}											
											});									
										}
									});
								}
							});
					
						break;
					
					case "addExit":
						var temp_id = req.body.id;				//id of the room we are adding exit to
						var temp_exit = req.body.roomExit;		//exit that we are adding..
						if(temp_exit === ""){
							res.render('editor.jade', {msg: "Error: Added room exit must not have empty field", rooms: oldRooms});
						}
						else{
						Room.findById(temp_id, function(err, data){	//find the room
								if (err){
									res.render('editor.jade', {msg: "Error: Unable to find room. This indicates a glitch in the server matrix...", rooms: oldRooms});
								}
								else
								{
									//add the tempExit to the roomExits array
									data.roomExits.push(temp_exit);
									//save this data.
									data.save(function(err){
										if(err){
											res.render('editor.jade', {msg: "Error: Unable to add room exit..", rooms: oldRooms});
										}
										else{
											//return user to the editor with new rooms
											Room.find(function(err,newRooms){
												if(err){
													res.redirect('/');
												}
												else{
													Room.findOne({name: temp_exit}, function(err,result){											
														if(result){	//if there is an existing room with name of the exit, give success message
															res.render('editor.jade', {msg: "Success! room exit added", rooms: newRooms});
														}
														else{
															res.render('editor.jade', {msg: "WARNING! The room designated as an exit does not exist. Consider adding this room or revising the exit that was just added.", rooms: newRooms});
														}									
													});
												
												}											
											});									
										}
									});
								}
							});
						}
						break;
						
					case "quit":
						res.redirect("/quit");
						break;
					
					default:
						res.redirect('/editor');
						break;
				}}	
			});

}



//myExport function fetches all rooms in the database, converts it to a JSON string, then renders it.
function myExport(req,res){
	Room.find({}, 'name title description roomExits', function(err, rooms){
			if (err || rooms === []) {
				res.render('export.jade', {exportedRooms: "No rooms to export.."});
			}
			else {
				var exportableRooms = JSON.stringify(rooms);
				res.render('export.jade', {exportedRooms: exportableRooms});
			}});
	
}


function myImport(req,res){
	//console.log("yolo debug");
	res.render('import.jade', {msg: ""});

}

var fs = require('fs'); 		//file reader
function importRooms(req,res){
	if(req.files){
		console.log("Import Received: ");
		//console.log(req.files);

		fs.readFile(req.files.playerRooms.path, "utf-8", function(err,data){
			if(err){
				console.log("error reading file.. " + err);
				res.render('import.jade', {msg: "Error reading your file..."});
			}
			else {
				console.log("The file uploaded is...");
				console.log(data);
				
				try{
					console.log("Parsing the rooms..");
					var importedRooms = JSON.parse(data); 
					console.log("Parsed the imported rooms!");
					console.log("Importing rooms!");
					emptyDB(function(){															//call emptyDB with the following callback
											
						importedRooms.forEach(function(value, index){							//for each object in the JSON array
							var newRoom = new Room({ 	name: value.name,						
														title: value.title,						
														description: value.description,
														roomExits: value.roomExits	});
							newRoom.save( function(err, room) { 								//save the room..
								if (err) {
									console.log("Error adding room: " + err);
								}else{
									console.log("room created.." + room.name + ", title: " + room.title);
								}
					})})}); 
					res.redirect('/');
				} 
				catch(e) {
					console.error("Parsing error: ", e);
					res.render('import.jade', {msg: "Error while parsing file, please check syntax."});
				}
			
			}			
		}); 

		
	}
	else{
		res.render('import.jade', {msg: "Error uploading your file, please try again."});
	}
	
	
}


exports.index = index;
exports.start = start;
exports.quit = quit;
exports.game = game;
exports.doAction = doAction;
exports.getContents = getContents;
exports.myImport = myImport;
exports.myExport = myExport;
exports.myEditor = myEditor;
exports.edit = edit;
exports.importRooms = importRooms;
